__author__ = 'TNF'

'''
This code retreives data from a symbol list (entered in the code, not read from file) where the first symbol
is the dependent variable in a linear regression and the remaining symbols in the list thr independent variables.
In this example, the ETFs, GLD and USO, are independent variables being used to model the price of the ETF, GDX.

The code retrieves the data, saves the data in a symbol-specific df, then merges the data (inner join) into a
single df (df_all).

The code then calculates StdDev, Kurtosis, and performs an ADF Stationarity Test.

Next, it runs a linear regression using the first symbol in the list as the dependent variable.
An ADF and Durbin Watson test is then performed on the residuals.

The code completes by generating a set of plots.  The plots can be turned on/off in two groups, Plot_Group1 and 2,
that are set as variables early in the code.

Plot_Group1:   Plots the time series data.
Plot_Group2:   Histogram of the data (check distribution if log returns).
Regression residuals are plotted by default.

The use case for this sample code is date retrival, analysis, and decision making in a single programmatic process.
This is an asset-lite example where the end user is not storing data, corp actions, writing bar builders,
split adjusters, etc.

This code runs against the Sandbox server is will be slower than the AWS load-balanced Producion environment.

'''


import pandas as pd
import io
import requests
import statistics
import statsmodels.tsa.stattools as ts
import statsmodels.api as sm
from statsmodels.stats.stattools import durbin_watson
from numpy import cumsum, log, polyfit, sqrt, std, subtract
from numpy.random import randn
import numpy as np
from numpy import *
from numpy import random
import datetime
import matplotlib.pyplot as plt

## API Credentials and Variables
api_user = "APIdemo"
api_pwd = "bACH93bH"
Start_Date = '01/01/2016'
End_Date = '01/15/2016'
Plot_Group1 = True
Plot_Group2 = True

## Define symbol set (place the DV first)
SymbolList = ['GDX', 'GLD', 'USO']
df = {}


## Request Data
'''

The url was built using Tick Data's RequestBuilder: https://sandbox-tickapi.tickdata.com/
Symbol, Start Data, and End Date have been parameterized.
One minute bar data has been specified in the url.

'''
for sym in SymbolList:

    ## Build the Sandbox url
    url =  'https://sandbox-tickapi.tickdata.com//stream?' \
           'COLLECTION_TYPE=COLLECTION_TYPE.US_TED' \
           '&EXTRACT_TYPE=COLLECTOR_SUBTYPE_US_TICK_TIMEBAR' \
           '&START_DATE=' + Start_Date + \
           '&END_DATE=' + End_Date + \
           '&DATE_FORMAT=MM/dd/yyyy' \
           '&TIME_FORMAT=HH:mm' \
           '&REQUESTED_DATA=' + sym + \
           '&SESSIONS=MARKET&GRANULARITY=1' \
           '&COMPRESSION_TYPE=NONE' \
           '&SELECTED_FIELDS=DATE_FIELD|TICK_TIME_FIELD|CLOSE_PRICE_FIELD'




    # Request the csv data with authentication
    r = requests.get(url, auth=(api_user,api_pwd)).content

    # Create a unique df by symbol
    df[sym] = pd.DataFrame()

    # Read the streamed csv data into the sym df
    df[sym] = pd.read_csv(io.StringIO(r.decode('utf-8')))

    # Reset the index=Date+Time
    df[sym] = df[sym].set_index(['Date', 'Time'])

    # Set df price column name to symbol
    df[sym].columns = [sym]

    print("Data Received For: ",sym)





## Conbine dfs.  Exclude any record not in all dfs.
df_alla = pd.merge(df[SymbolList[0]], df[SymbolList[1]], left_index=True, right_index=True)
df_all= pd.merge(df_alla, df[SymbolList[2]], left_index=True, right_index=True)
print('\n',df_all.head())




#### --- Begin Analytics
###
print('\n',"StdDev, Kurtosis, ADF Stationarity Test:")

# Calc Std Deviations, kurtosis, and ADF of each time series
for sym in SymbolList:
    stdDev = np.std(df_all[sym].values)
    print(sym)
    print(stdDev , df_all[sym].kurt())
    print(ts.adfuller(df_all[sym], 1))


# Test 1: Check for stationarity  Augmented Dickey-Fuller (ADF) Test
print()
def check_for_stationarity(X, cutoff=0.01):
    # We must observe significant p-value to convince ourselves that the series is stationary
    pvalue = ts.adfuller(X)[1]
    if pvalue < cutoff:
        print('p-value = ' + str(pvalue) + ' The series ' + X.name +' is likely stationary.')
        return True
    else:
        print('p-value = ' + str(pvalue) + ' The series ' + X.name +' is likely non-stationary.')
        return False


## Check stationarity of each time series by calling the ADF test
print("ADF Test P-values: ")
for sym in SymbolList:
    check_for_stationarity(df_all[sym])



### --- Regressions
'''
http://www.statsmodels.org/stable/regression.html#examples
http://www.statsmodels.org/stable/generated/statsmodels.regression.linear_model.RegressionResults.html
'''
## Define y and X variables for the regression
y = df_all['GDX']
X = df_all[['GLD','USO']]


# Generate Regression
def regression_results(y, X):
    X = sm.add_constant(X)              # specify a model with an intercept
    model_type = sm.OLS(y, X)           # define a model type
    model = model_type.fit()            # create an instance of that model type named 'model'
    std_err = math.sqrt(model.scale)    # Get regression stats
    print(model.summary())              # Coefficients

    # Regression and Residuals
    df_all['pred'] = model.fittedvalues         # Get model prediction
    df_all['residuals'] = model.resid * -1      # Get residuals. Change sign to get to Pred-DV.
    df_all['residualZ'] = df_all['residuals'] / std_err
    print()

regression_results(y,X)




#### --- Analyze Residuals

## Check for Stationarity
print('\n','ADF Test on Residuals')                       # Is the same for ResidualZ
check_for_stationarity(df_all['residuals'])

## Check for autocorrelation (serial correlation)
'''
The Durbin Watson test reports a test statistic, with a value from 0 to 4, where:
2 is no autocorrelation.
0 to <2 is positive autocorrelation (common in time series data).
>2 to 4 is negative autocorrelation (less common in time series data).
'''
print('\n','Durbin Watson Test for Autocorrelation 0-4')
print(durbin_watson(df_all['residuals']))

# Durbin Watson Decay plot
# plt.plot(ts.acf(df_all['residuals']))
# plt.show()





#### ---Plots
## Line plot of the data as Figure 1.  Share X axis.
if Plot_Group1:
        with plt.style.context('ggplot'):
            plt.figure(1,figsize=(12,8))

            ax1 = plt.subplot(311)
            df_all[SymbolList[0]].plot(title=SymbolList[0], color='b')

            ax2 = plt.subplot(312, sharex=ax1)
            df_all[SymbolList[1]].plot(title=SymbolList[1], color='b')

            ax3 = plt.subplot(313, sharex=ax1)
            df_all[SymbolList[2]].plot(title=SymbolList[2], color='b')

        plt.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)         # Force plots to fit
        plt.show()


## Plot Residuals
with plt.style.context('ggplot'):
    plt.figure(3,figsize=(12,8))
    ax5 = plt.subplot(111)
    df_all['residuals'].plot(title='Residuals', color='b')
plt.show()


## Basic histograms as Figure 2
if Plot_Group2:
    with plt.style.context('ggplot'):
        plt.figure(2,figsize=(12,8))

        ax1 = plt.subplot(311,title=SymbolList[0])
        df_all[SymbolList[0]].hist(bins=50)

        ax2 = plt.subplot(312, title=SymbolList[1])
        df_all[SymbolList[1]].hist(bins=50)

        ax3 = plt.subplot(313, title=SymbolList[2])
        df_all[SymbolList[2]].hist(bins=50)

    plt.show()







